/**
 * Description:  WPF builder regen class
 *
 * Licensed Materials - Property of IBM 
 * 5724-O03
 * (C) Copyright 2006. IBM Corp. All rights reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * See WebSphere Portlet Factory ( product id 5724-O03 ) product license for terms and conditions of use. 
 *
 * This Java source file was generated by the WebSphere Portlet Factory.
 *
 * Do not edit generated code within the following comments /*##GENERATED_...
 */
package com.bowstreet.builders.webapp;

import com.bowstreet.builders.webapp.foundation.WebAppBuilder;
import com.bowstreet.generation.BuilderCall;
import com.bowstreet.generation.BuilderInputs;
import com.bowstreet.generation.GenContext;
import com.bowstreet.model.VariableNotFoundException;
import com.bowstreet.util.IXml;
import com.bowstreet.util.StringUtil;
import com.bowstreet.webapp.Variable;
import com.bowstreet.webapp.WebApp;

/**
 * Sets the persist flag on one or more variables to a specified value.
 */
public class VariableScopeSetterBuilder implements WebAppBuilder
{
    /** Maps the Variable.PERSIST_* values to the names used by the builder. */
    public static final String scopeNames[] = {
        "session", //$NON-NLS-1$ // PERSIST_STATEFUL
        "no_failover", //$NON-NLS-1$ // PERSIST_NOTPERSISTED
        "readonly", //$NON-NLS-1$ // PERSIST_READONLY
        "request", //$NON-NLS-1$ // PERSIST_REQUEST_SCOPE
    };
    /**
     * This is the method that's called during generation of the WebApp.
     */
    public void doBuilderCall(GenContext genContext, WebApp webApp, BuilderCall builderCall, BuilderInputs builderInputs)
    {
        // Generated code to get all the builder inputs
        IXml variables = builderInputs.getXml(Constants.Variables, null);       
      
        // Arrange to do real work in post-construction, when all variables should be available.
        if (genContext.getGenerationPhase().equals(GenContext.PHASE_CONSTRUCTION))
        {
            // If no variables have been specified, no need to defer.
            if (variables != null)
                genContext.deferBuilderCall(GenContext.PHASE_POSTCONSTRUCTION, builderCall);
            return;
        }
        
        // Process all variables specified in the builder UI.
        for (IXml variable = variables.getFirstChildElement(); variable != null; variable = variable.getNextSiblingElement())
        {
            // Validate that the named variable exists.
            String variableName = variable.getText("Name"); //$NON-NLS-1$
            Variable var = webApp.getVariable(variableName);
            if (var == null)
            {
                if (genContext.isDesignTime())
                    throw new VariableNotFoundException(variableName);
                else
                    continue;
            }
            // Set the scope.
            String scope = variable.getText("Scope"); //$NON-NLS-1$
            if (!setScope(var, scope))
                builderCall.addMessage(BuilderCall.SEVERITY_ERROR, "Invalid scope " + scope + " for variable " + var.getName());

            // Set the instantiator.
            String instantiator = variable.getText("Instantiator"); //$NON-NLS-1$
            if (!setInstantiator(var, instantiator, webApp))
                builderCall.addMessage(BuilderCall.SEVERITY_ERROR, "Invalid instantiate action " + instantiator + " for variable " + var.getName());                    
        }
    }
    
    /*
     * Set the scope for the specified variable, if one is supplied. Return false if an invalid scope was specified.
     */
    private static boolean setScope(Variable var, String scope)
    {
        if (StringUtil.isEmpty(scope) || scope.equals("do_not_change")) //$NON-NLS-1$
            return true;
        
        for (int persistFlag = 0; persistFlag < scopeNames.length; persistFlag++)
        {
            if (scopeNames[persistFlag].equals(scope))
            {
                var.setPersistFlag(persistFlag);
                return true;
            }
        }
        return false;
    }

    /*
     * Set the "instantiate via" method for the variable, returning true if everything went OK.
     */
    private static boolean setInstantiator(Variable var, String instantiator, WebApp webApp)
    {
        if (StringUtil.isEmpty(instantiator))
            return true;
        if (webApp.getAction(instantiator) != null)
        {
            var.setInstantiateMethod(instantiator);
            return true;
        }
        return false;
    }

    /**
     * Interface that has constants for all the builder input names
     */
    static public interface Constants
    {
        /*##GENERATED_BEGIN*/
        public static final String Name = "Name"; //$NON-NLS-1$
        public static final String Variables = "Variables"; //$NON-NLS-1$
        public static final String AddAllVariablesButton = "AddAllVariablesButton"; //$NON-NLS-1$
        /*##GENERATED_END*/
    }
}
