/**
 * Description:  WPF builder regen class
 *
 * Licensed Materials - Property of IBM 
 * 5724-O03
 * (C) Copyright 2006. IBM Corp. All rights reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * See WebSphere Portlet Factory ( product id 5724-O03 ) product license for terms and conditions of use. 
 *
 * This Java source file was generated by the WebSphere Portlet Factory.
 *
 * Do not edit generated code within the following comments /*##GENERATED_...
 */
package com.bowstreet.builders.webapp.ui;


import java.util.Iterator;

import com.bowstreet.builders.webapp.VariableScopeSetterBuilder;
import com.bowstreet.builders.webapp.VariableScopeSetterBuilder.Constants;
import com.bowstreet.editor.uitools.coordinator.WebAppBaseCoordinator;
import com.bowstreet.generation.DynamicBuilderInputDefinition;
import com.bowstreet.util.IXml;
import com.bowstreet.util.StringUtil;
import com.bowstreet.util.XmlUtil;
import com.bowstreet.webapp.Variable;
import com.bowstreet.webapp.WebApp;

/**
 * Coordinator for the Variable Scope Setter builder.
 */
public class VariableScopeSetterCoordinator extends WebAppBaseCoordinator
{
    /*
     * @see com.bowstreet.editor.Coordinator#processInputChange(com.bowstreet.generation.DynamicBuilderInputDefinition)
     */
    public boolean processInputChange(DynamicBuilderInputDefinition changed)
    {
        if (changed == null)
            return false;
        
        if (changed.getName().equals(Constants.Variables) || changed.getName().equals(Constants.AddAllVariablesButton))
        {
            // Handle two main cases here: user added a new variable, or user hit the "add all variables" button.
            // We don't try to check for the other cases that come in here (e.g., user changes variable scope)
            // because data won't be modified below.
            boolean modifiedData = false;
            IXml variables = context.findInputDefinition(Constants.Variables).getXml();
            if (variables == null)
            {
                variables = XmlUtil.create("Variables"); //$NON-NLS-1$
            }
            if (changed.getName().equals(Constants.AddAllVariablesButton)) //$NON-NLS-1$
            {
                // Look up all variables in the webapp and add them to the builder input structure.
                modifiedData = addAllVariables(getWebApp(), variables);  
            }
            // Make sure that all variables in the builder input have appropriate values for scope and instantiator.
            if (setCurrentVariableDetails(getWebApp(), variables))
            {
                modifiedData = true;
            }
            
            // If we changed the data, clone it and set it back.
            if (modifiedData)
            {
                context.findInputDefinition(Constants.Variables).setXml(variables.cloneElement());
            }
            
            return modifiedData;
        }
        return false;
    }

    // Make sure that the variables structure contains an entry for every variable in the webapp.
    private boolean addAllVariables(WebApp webApp, IXml variables)
    {       
        boolean addedVariable = false;
        if (variables == null)
            return addedVariable;        
        for (Iterator vars = webApp.getVariables(); vars.hasNext();)
        {
            Variable var = (Variable)vars.next();
            String name = var.getName();
            IXml entry = variables.findElement("//Variable/[Name=" + name + "]"); //$NON-NLS-1$ //$NON-NLS-2$            
            if (entry != null)
                continue;
            
            variables.addChildElement("Variable").setText("Name", name); //$NON-NLS-1$ //$NON-NLS-2$
            addedVariable = true;
        }
        return addedVariable;
    }

    // Make sure that any variables without a selected scope/instantiator have the current value set.
    private boolean setCurrentVariableDetails(WebApp webApp, IXml variables)
    {
        boolean changed = false;
        if (variables == null)
            return changed;
        for (IXml variable = variables.getFirstChildElement(); variable != null; variable = variable.getNextSiblingElement())
        {
            // If no name, we're done.
            String variableName = variable.getText("Name"); //$NON-NLS-1$
            if (StringUtil.isEmpty(variableName))
                continue;
                        
            // See if the variable exists in the WebApp.
            Variable var = webApp.getVariable(variableName);
            if (var == null)
                continue;       

            // Is there an assigned scope?
            String scope = variable.getText("Scope"); //$NON-NLS-1$
            if (StringUtil.isEmpty(scope))
            {
                // No scope: get the persist flag from the variable and map it to the builder scope names.
                int persistFlag = var.getPersistFlag();
                if (persistFlag < 0 || persistFlag >= VariableScopeSetterBuilder.scopeNames.length)
                    continue;
                scope = VariableScopeSetterBuilder.scopeNames[persistFlag];
    
                // Update the XML with the current scope.
                variable.setText("Scope", scope); //$NON-NLS-1$
                changed = true;
            }
            // Similarly, make sure the instantiator field has a value, either from the
            // builder UI or from the WebApp.
            String instantiator = variable.getText("Instantiator"); //$NON-NLS_1$
            if (StringUtil.isEmpty(instantiator))
            {
                instantiator = var.getInstantiateMethod();
                if (!StringUtil.isEmpty(instantiator))
                    variable.setText("Instantiator", instantiator); //$NON-NLS-1$
            }
        }
        return changed;
    }
}
